"""

epXML2CSV

A utility to extract numbers from the tabular XML file
created by EnergyPlus and put them into a CSV format.
The utility is especially useful when extracting data from
a large number of simulation results. The utility collects
values from all XML files in the directory.

  epXML2CSV csvfilename  extractfilename

A row is generated for each xmlfile in the directory.
The columns correspond to each item extracted.

The extractfilename format is shown below and each row of the file
corresponds to one column of the resulting CSV file.

userHeading, reportNameString, forString, subReportString, nameString, headingString
userHeading, reportNameString, forString, subReportString, nameString, headingString
userHeading, reportNameString, forString, subReportString, nameString, headingString
...etc..

If the forString is * then new tables are generated at the end of the file for each
consisting of the one line for each file and one column for each possible forString
found across the files.

Copyright (c) 2014 Jason Glazer

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

"""

import xml.etree.cElementTree as ET
import sys
import os
import string
import csv
from pprint import pprint
from sets import Set

# Main function that returns a string based on the extract info
def getTextFromEPXML(root, reportS, forS, subRepS, nameS, headS):
    for repEl in root.findall('./' + reportS):
        if repEl.find('for').text == forS:
            for subRepEl in repEl.findall('./' + subRepS):
                if subRepEl.find('name').text == nameS:
                    try:
                        resultString = subRepEl.find(headS).text
                    except:
                        resultString = 'NAME NOT FOUND'
                    if resultString != None:
                        return resultString
    return "NOT FOUND"

def epXML2CSV(argv=None):
    if argv == None:
        argv = sys.argv
    # get the command line arguments
    [progName, csvFileName, extractFileName] = sys.argv

    # AR: Might be good to check for well-formed-ness of input parameters,
    # existence of files, etc. but if this is a script that only you will
    # use, I won't harp.

    extractLines = []
    extractFile = open(extractFileName, 'r')
    extractReader = csv.reader(extractFile, delimiter=',')
    for row in extractReader:
        if row:
            if row[0][0] != "#":  #remove rows that start with comment symbol "#"
                extractLines.append(map(string.strip, row))

    extractFile.close()
    csvFile = open(csvFileName, 'wb')
    csvWriter = csv.writer(csvFile, delimiter=',')

    firstHeadings = []
    listOfPulledUsingForWild = []
    userHeadingForWild = []
    for xmlFileName in os.listdir("."):
        if xmlFileName.endswith(".xml"):
            # The following lists will contain the output for the CSV file, starting with name of XML file
            Headings = ["FileName"]
            Pulled = [xmlFileName]
            # the heavy lifting of parsing the XML file is done on the next line by the ElementTree library
            tree = ET.parse(xmlFileName)
            root = tree.getroot()

            # -----------  Get the header information -------------
            #  <BuildingName>Building</BuildingName>
            #  <EnvironmentName>Chicago Ohare Intl Ap IL USA TMY3 WMO#=725300</EnvironmentName>
            #  <WeatherFileLocationTitle>Chicago Ohare Intl Ap IL USA TMY3 WMO#=725300</WeatherFileLocationTitle>
            #  <ProgramVersion>EnergyPlus-Windows-32 8.0.0.008, YMD=2013.05.16 11:07</ProgramVersion>
            #  <SimulationTimestamp>
            #    <Date>
            #      2013-05-16
            #    </Date>
            #    <Time>
            #      11:07:53
            #    </Time>
            #  </SimulationTimestamp>

            Headings.append('BuildingName')
            Pulled.append(root.find('BuildingName').text)

            Headings.append('EnvironmentName')
            Pulled.append(root.find('EnvironmentName').text)

            Headings.append('WeatherFileLocationTitle')
            Pulled.append(root.find('WeatherFileLocationTitle').text)

            Headings.append('ProgramVersion')
            Pulled.append(root.find('ProgramVersion').text.replace(","," "))

            Headings.append('SimulationTimestamp')
            Pulled.append(root.find('SimulationTimestamp').find('Date').text.strip() + ' ' + root.find('SimulationTimestamp').find('Time').text.strip())

            for [userHeadingStr, reportStr, forStr, subReportStr, nameStr, headingStr] in extractLines:
                #when using a wildcard on the FOR an extra column is needed one for the name of each FOR
                if forStr == '*':
                    #loop through all instances of the report for each "for"
                    forFounds = []
                    PulledFromFor = {}
                    for repEl in root.findall('./' + reportStr):
                        forFounds.append(repEl.find('for').text)
                    for forFound in forFounds:
                        PulledFromFor[forFound] = getTextFromEPXML(root, reportStr, forFound, subReportStr, nameStr, headingStr)
                    listOfPulledUsingForWild.append([xmlFileName,userHeadingStr,PulledFromFor])
                    if userHeadingStr not in userHeadingForWild:
                        userHeadingForWild.append(userHeadingStr)
                    Pulled.append('SEE BELOW') # a filler to make sure a space is taken
                else:
                    Pulled.append(getTextFromEPXML(root, reportStr, forStr, subReportStr, nameStr, headingStr))
                Headings.append(userHeadingStr)

            if Headings != firstHeadings:
                firstHeadings = Headings
                csvWriter.writerow(Headings)

            csvWriter.writerow(Pulled)

            print "File: " + xmlFileName + "   Number of values: " + str(len(Pulled))
 #   pprint(listOfPulledUsingForWild,stream=csvFile)
    for userHeading in userHeadingForWild:
        #create a table for each user defined heading with columns of FOR's and rows of the filenames
        csvFile.write('\n') #leave a blank line
        csvFile.write('\n') #leave a blank line
        csvFile.write(userHeading + '\n')
        prevKeys = []
        for [file,userHeadingInList,resDict] in listOfPulledUsingForWild:
            if userHeading == userHeadingInList:
                curKeys = resDict.keys()
                if curKeys != prevKeys:  #only print a new list of
                    csvFile.write('\n') #leave a blank line
                    row = ['File',]
                    row.extend(curKeys)
                    csvWriter.writerow(row)
                    prevKeys = curKeys
                row = []
                row.append(file)
                for curKey in curKeys:
                    row.append(resDict[curKey])
                csvWriter.writerow(row)
    csvFile.close()


if __name__ == "__main__":
    sys.exit(epXML2CSV())
